//----------------------------------------------------------------------------
// 
//	file: matrix_window.c
// 
//	function: grafix window class extension for matrix display 
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Mon Jul  6 15:23:25 CDT 1998
// 
//	$Revision: 1.7 $ $Date: 2001/11/30 20:24:15 $
//	$Log: matrix_window.cc,v $
//	Revision 1.7  2001/11/30 20:24:15  yschoe
//	pglissom v1.0 release
//
//	Revision 1.6  2000/02/13 10:29:43  yschoe
//	*** empty log message ***
//
//	Revision 1.5  1999/12/22 05:29:48  yschoe
//	draw circle, adding pointer to mult_matrix .
//
//	Revision 1.4  1998/11/17 12:43:25  yschoe
//	added mouse event mask
//
//	Revision 1.3  1998/10/08 05:48:29  yschoe
//	fixed coordinates : screen coord to cartesian coord
//	(0,0) +-----            |
//	      |                 |
//	      |         to      |
//	      |            (0,0)+-------
//
//	Revision 1.2  1998/09/04 03:36:37  yschoe
//	fixed all warnings generated by -Wall
//
//	Revision 1.1  1998/07/23 01:07:05  yschoe
//	jul22 stable version (except netlayer)
//
//
//----------------------------------------------------------------------------

#include <window.h>
#include <string.h>
#include <iostream.h>
#include <stream.h>
#include "matrix_window.h"
#include "ppmout.h"

//----------------------------------------------------------------------------
//	matrix_window() 
//	  - Constructors
//	  - assumes that the matrix linked to this class is already
//		malloced, and is in column-major (traverses column first)
//	  - assumes coordinate is in cartesian, not in Xwin coordinate
//----------------------------------------------------------------------------
matrix_window::matrix_window
	(window &parent, int w, int h, int x, int y, int bw,
	int color_range, int col, int row)
: pixmap_window(parent,w,h,x,y,bw) {

  selection_mask |= PointerMotionMask | ButtonPressMask | ButtonReleaseMask;

  init(color_range,COLOR,col,row);

  stats = NULL;

}

//----------------------------------------------------------------------------
matrix_window::matrix_window
	(window &parent, int w, int h, int x, int y, int bw,
	int color_range, int col, int row, REAL *oned)
: pixmap_window(parent,w,h,x,y,bw) {

  init(color_range,COLOR,col,row);

  dim_mode = OneD;
  matrix_window::oned = oned; // link to actual matrix

  stats = NULL;

}

//----------------------------------------------------------------------------
matrix_window::matrix_window
	(window &parent, int w, int h, int x, int y, int bw,
	int color_range, int col, int row, REAL **twod)
: pixmap_window(parent,w,h,x,y,bw) {

  init(color_range,COLOR,col,row);

  dim_mode = TwoD;
  matrix_window::twod = twod; // link to actual matrix

  stats = NULL;

}

//----------------------------------------------------------------------------
matrix_window::matrix_window
	(window &parent, int w, int h, int x, int y, int bw,
	int color_range, int color_mode, int col, int row)
: pixmap_window(parent,w,h,x,y,bw) {

  matrix_window::color_mode = (ColMode) color_mode;
  init(color_range,COLOR,col,row);

  stats = NULL;

}

//----------------------------------------------------------------------------
matrix_window::matrix_window
	(window &parent, int w, int h, int x, int y, int bw,
	int color_range, int color_mode, int col, int row, REAL *oned)
: pixmap_window(parent,w,h,x,y,bw) {

  init(color_range,color_mode,col,row);

  dim_mode = OneD;
  matrix_window::oned = oned;

  stats = NULL;

}

//----------------------------------------------------------------------------
matrix_window::matrix_window
	(window &parent, int w, int h, int x, int y, int bw,
	int color_range, int color_mode, int col, int row, REAL **twod)
: pixmap_window(parent,w,h,x,y,bw) {

  init(color_range,color_mode,col,row);

  dim_mode = TwoD;
  matrix_window::twod = twod;

  stats = NULL;

}

//----------------------------------------------------------------------------
//	~matrix_window() 
//	  - Destructor
//----------------------------------------------------------------------------
matrix_window::~matrix_window(){

  XFreePixmap(display,pix);
  delete [] color_map;
  // No need to free *oned or **twod : it might be used further.

}

//----------------------------------------------------------------------------
//	init() 
//	  - init common values
//----------------------------------------------------------------------------
void matrix_window::init(int color_range, int color_mode, int col, int row) {

  matrix_window::color_mode = (ColMode) color_mode; // default clr mode = COLOR
  matrix_window::scale = 1.0; // default scale

  matrix_window::col  = col;  // columns in matrix
  matrix_window::row  = row;  // rows in matrix
  matrix_window::color_range = color_range; // number of colors to use
  color_map = new int[color_range]; // allocate table for cmap

  cerr << "matrix_window::init color_range = " << color_range << endl;

  // Actually allocate colors
  alloc_colors();

  // Init buttons/slides
  init_gadgets();
  
}

//----------------------------------------------------------------------------
//	draw_interior() 
//	  - main drawing routine : use redraw() when you need to invoke this
//----------------------------------------------------------------------------
void matrix_window::draw_interior() {

  int i, xgran, ygran;
  // DEBUG: printf("matrix_window::draw_interior()\n");

  xgran = width/col;
  ygran = height/row;

  int cells=col*row;
  for (i=0; i<cells; ++i)
  {
	// sets gc_copy 
	if (dim_mode == OneD) {
           set_color(color_map[MIN((int)(oned[i]*scale*color_range),
			           color_range-1)]); 
	} else if (dim_mode == TwoD) {
       	   set_color(color_map[MIN((int)(twod[i%col][i/col]*scale*color_range),
				   color_range-1)]); 
	}
	// actually draws the intensity
        XFillRectangle(display,pix,gc_copy,
		i%col*xgran,(row-i/col-1)*ygran,xgran,ygran);

	// draw row tick
	if (i%col==0) {
		REAL length;
		if ((i/col)%5==0) {
			length = xgran;	
		} else {
			length = xgran*0.5;
		}			
		set_color(white);
		XDrawLine(display,pix,gc_copy,
			(int)(i%col*xgran), (int)((row-i/col-1+0.5)*ygran),
			(int)(i%col*xgran+length),(int)((row-i/col-1+0.5)*ygran));
	}
	// draw column tick
	if ((i/col)==0) { 
		REAL length;
		if ((i%col)%5==0) {
			length = ygran;	
		} else {
			length = ygran*0.5;
		}		

		set_color(white);
		XDrawLine(display,pix,gc_copy,
			(int)((i%col+0.5)*xgran),
			(int)((row-i/col-1+1)*ygran),
		    	(int)((i%col+0.5)*xgran),
			(int)((row-i/col-1+1)*ygran-length));
	}
  }
  // set_color(red);
  // DrawString(20,20,"Greetings : matrix window");
  set_color(black);
  XDrawRectangle(display,pix,gc_copy,0,0,col*xgran,row*ygran);

  // Draw simstat objects : if stats* is not NULL -- user needs to
  // call matrix_widow::register_stat()
	
  if (stats != NULL)  { // sanity check
	  for (i=0; i<stats->get_num_specs(); ++i)  {
		switch(stats->get_type(i)) {
		case SS_CIRCLE :  // circle spec
			draw_circle(stats->get_cx(i),stats->get_cy(i),
				stats->get_cradius(i),1.0);
			break;
		case SS_SQR:     // square spec
			draw_square(stats->get_cx(i),stats->get_cy(i),
				stats->get_radius(i));
			break;
		case SS_BBOX:    // bounding box (rectangle) spec
			draw_rectangle( stats->get_lowx(i), stats->get_lowy(i),
				stats->get_highx(i), stats->get_highy(i));
			break;
	       }
	  }
  }

  XFlush(display);
}

//----------------------------------------------------------------------------
//	set_scale() 
//	  - change the scale
//----------------------------------------------------------------------------
void matrix_window::set_scale(REAL s) {
  
  scale = s;
  redraw();

}

//----------------------------------------------------------------------------
//	alloc_colors() 
//	  - allocate colors -- BW or COLOR
//----------------------------------------------------------------------------
void matrix_window::alloc_colors() {
 
  // Initialize color_map according to color_mode.
  int	color_gran = 0xffff/color_range;
  int	one_third_range = color_range/3;

  // Actually allocate the necessary color
  if (color_mode == BW) {
	for (int i=0; i<color_range; ++i) {
		int v = i*0xffff/color_range;
	 	color_map[i] = alloc_color(v,v,v);
	}
  } else if (color_mode == COLOR) {
	for (int i=0; i<one_third_range; ++i) {
		color_map[i] 
			= alloc_color(color_gran*3*i,0,0);	
		color_map[one_third_range+i] 
			= alloc_color(0xffff,color_gran*3*i,0);	
		color_map[2*one_third_range+i] 
			= alloc_color(0xffff,0xffff,color_gran*3*i);	
	}
	color_range = one_third_range * 3; // adjust color_range
  } else if (color_mode == HSV_MULT || color_mode == HSV) {
	for (int i=0; i<color_range; ++i) {
	   if (i==0 && color_mode == HSV_MULT) {
			color_map[0] = alloc_color(0xffff,0xffff,0xffff);
	   } else {
		REAL v = i/(REAL)color_range;
		int r, g, b;
		hsvrgb::hsv2rgb(v,1.0,1.0,&r,&g,&b);
		color_map[i] = alloc_color(r*256,g*256,b*256);
	   }
	}
  }
}


//----------------------------------------------------------------------------
//	init_gadgets() 
//	  - start buttons, slides, etc
//----------------------------------------------------------------------------
void matrix_window::init_gadgets() {

  //menu_bar *mb = new menu_bar(*mainw,width,20,0,0);
  //new delete_button(*mb,mainw,width/5,20,0,0);

  // Let's make this class as generic as possible and add 
  // stuff later in a derived class.

}


//----------------------------------------------------------------------------
//	disp_values() 	: let's not use this
//	  - dump matrix values to a text array
//----------------------------------------------------------------------------
char **matrix_window::disp_values() {

  char **dump = new char* [row];
  char *buf  = new char [50];

  int cells=col*row;

  for (int i=0; i<row; ++i) {
	dump[i] = new char [col*10+1];	// 10 chars per value, and \n at end
  	strncpy(dump[i],"",0); // clear string
  }

  for (int i=0; i<cells; ++i)
  {
	if (dim_mode == OneD) {
		sprintf(buf,"%.4f ",oned[i]);
		strncat(dump[i/col],buf,strlen(buf));
		printf("%s",buf);
		if (i%col==0) printf("\n");
	} else if (dim_mode == TwoD) {
		sprintf(buf,"%.4f ",twod[i%col][i/col]);
		strncat(dump[i/col],buf,strlen(buf));
		printf("%s",buf);
		if (i%col==0) printf("\n");
	}
  }

  // delete [] dump;
  return dump;
}


//----------------------------------------------------------------------------
//	BRelease_CB() 
//	  - button press event
//----------------------------------------------------------------------------
void matrix_window::BPress_CB(XButtonEvent ev) {

  cout << "BPress" << endl;

}

void matrix_window::Motion_CB(XMotionEvent ev) {

  cout << "Motion" << endl;

}

void matrix_window::BRelease_CB(XButtonEvent ev) {

  int xgran, ygran;

  xgran = width/col;
  ygran = height/row;

  xprev = ev.x;
  yprev = ev.y;
  
  colprev = ev.x/xgran;
  rowprev = (height-ev.y)/ygran;

  cout << "matrix window : clicked on   = (" 
       << xprev << "," << yprev << ")" << endl;
  cout << "matrix window : matrix coord = (" 
       << colprev << "," << rowprev <<")" << endl;

}
//----------------------------------------------------------------------------
//	draw_circle() 
//	  - draw a circle
//----------------------------------------------------------------------------
void matrix_window::draw_circle(int cx, int cy, int radius, REAL val) {

  REAL xgran, ygran;

  xgran = width/col;
  ygran = height/row;

  // set coordinates
  int x = (int)((cx-radius)*xgran)+0;
  int y = int((row-1)*ygran)-(int)((cy+radius)*ygran);

  int w = (int)(radius*2.0*xgran);
  int h = (int)(radius*2.0*ygran);

  // set_color(color_map[MIN((int)(val*scale*(color_range-1)),color_range-1)]); 
  if (color_mode == HSV_MULT) {
  	set_color(black);
  } else {
	set_color(white);
  }
  // XSetLineAttributes(display,gc_copy,2,LineSolid,CapButt,JoinMiter);
  XDrawArc(display,Win,gc_copy,x+int(xgran/2.0),y+int(ygran/2.0),w,h,
	0,
	int(64*360));
  // XSetLineAttributes(display,gc_copy,1,LineSolid,CapButt,JoinMiter);
}

//----------------------------------------------------------------------------
//	draw_rectangle() 
//	  - draw a rectangle 
//----------------------------------------------------------------------------
void matrix_window::draw_rectangle(int x1, int y1, int x2, int y2) {

  REAL xgran, ygran;

  xgran = width/col;
  ygran = height/row;

  // set coordinates
  int sx = (int)(x1*xgran)+0;
  int sy = int((row-1)*ygran)-(int)(y2*ygran);

  int w = (int)((x2-x1+1)*xgran);
  int h = (int)((y2-y1+1)*ygran);

  // set_color(color_map[MIN((int)(val*scale*(color_range-1)),color_range-1)]); 
  if (color_mode == HSV_MULT) {
  	set_color(black);
  } else {
	set_color(white);
  }
  // XSetLineAttributes(display,gc_copy,2,LineSolid,CapButt,JoinMiter);
  XDrawRectangle(display,Win,gc_copy,sx,sy,w,h);
  // XSetLineAttributes(display,gc_copy,1,LineSolid,CapButt,JoinMiter);
}

//----------------------------------------------------------------------------
//	draw_square() 
//	  - draw a square
//----------------------------------------------------------------------------
void matrix_window::draw_square(int cx, int cy, int radius) {

  draw_rectangle(cx-radius, cy-radius, cx+radius, cy+radius);
	
}

//----------------------------------------------------------------------------
//	add_mult_matrix() 
//	  - add pointer to matrix to multiply
//----------------------------------------------------------------------------
void matrix_window::add_mult_matrix(REAL **mult_mat) {

  matrix_window::mult_mat  = mult_mat;

}
