## usage: [E,V,Eimg,Simg,g] = edgemat(Img, gridsize, freq)
##
##	Img: p x q matrix (grayscale image)
##
## Returns a n x m matrix E where 
##	n = p / gridsize
##	m = q / gridsize
##	and each E_ij is the edge orientation of the subimage patch.
##
## First, convolve with DoG filter, and then convolve each
## image patch of size gridsize x gridsize with gabor filters of
## orientations 0, 10, 20, ... , 180.	

## Author: Yoonsuck Choe <choe@tamu.edu>

function [E,V,Eimg,Simg,g] = edgemat(Img, gridsize, freq)

	# get image size
	[oldp, oldq] = size(Img);

	# adjust grid size to have the conv kernels
	# centered at the middle: i.e. make the gridsize an odd number.
	if ( rem(gridsize,2) == 0 ) 
		gridsize ++;
	endif

	DoG = dog(gridsize);

 	# convolve with DoG, and only use the real part
	Eimg = real(conv2(Img,DoG,"same"));	

	# calculate new image size to make it divisible by gridsize
	p = floor(oldp/gridsize)*gridsize;
	q = floor(oldq/gridsize)*gridsize;

   	ygrids = floor(p/gridsize);
	xgrids = floor(q/gridsize);

	# get subimages: each row vector is the subimage
	# you can visualize the subimages with pcavecview(Simg');

	Simg = samplemat(Eimg,gridsize,gridsize);

	# pcavecview(Simg',xgrids * ygrids);
	
	# precalculate Gabor functions of size gridsize x gridsize
	# and store them as row vectors
	g = vec(gabor(gridsize,gridsize,"sin",0,freq))';

	for i=(10:10:180)
		gtmp = vec(gabor(gridsize,gridsize,"sin",i,freq))';
		g = [g ; gtmp];
	endfor;

	#pcavecview(g',19);

	# calc dotproduct
	V = zeros(ygrids,xgrids).-999999;
	E = zeros(ygrids,xgrids);

	for i=1:ygrids
		for j=1:xgrids
			idx = j + (i-1)*xgrids;	
			for k=1:19
				dtmp = g(k,:)*Simg(idx,:)';
				if (V(i,j) < dtmp) 
					V(i,j)=dtmp;
					E(i,j) = k;
				endif
			endfor
		endfor
	endfor

endfunction
